clear; clc; % close all;

% ----- Grids ----- %
Z = 1;
N = 512;
M = 70;
L = 2e-3*2;
Lout = 7.5e-3*2; 
[xsrc,ysrc] = meshgrid((-N/2:N/2-1)*L/N);
[xs,ys] = meshgrid((-M/2:M/2-1)*L/M);
[fxs,fys] = meshgrid((-M/2:M/2-1)*1/L);
fxs = repmat(fxs,[1 1 M^2]);
fys = repmat(fys,[1 1 M^2]);
[x1,y2] = meshgrid((-N/2:N/2-1)*L/N);
[y1,x2] = meshgrid((-N/2:N/2-1)*L/N);
[x1a,x2a] = meshgrid((-N/2:N/2-1)*L/N);
[~,y1a] = meshgrid((-N/2:N/2-1)*L/N);
% ----------------- %

% ----- TAGSM Parameters ----- %
sigma = 41*(1e-3)^(-2);
alpha = 0.1*(1e-3)^2;
beta = 0.2*(1e-3)^2;
u = -20*(1e-3)^(-2); 
a = alpha*u;
b = beta*u;
if alpha < beta
    if u >= 1/alpha + sqrt((beta-alpha)/(alpha^2*beta)) || ...
            u <= 1/alpha - sqrt((beta-alpha)/(alpha^2*beta))        
    else
        error(['Nonphysical u: u = ' num2str(u) ' u >= ' num2str(1/alpha + sqrt((beta-alpha)/(alpha^2*beta))) ...
            ' or u <= ' num2str(1/alpha - sqrt((beta-alpha)/(alpha^2*beta)))]);
    end
elseif alpha > beta
    if u >= 1/beta + sqrt((alpha-beta)/(alpha*beta^2)) || ...
            u <= 1/beta - sqrt((alpha-beta)/(alpha*beta^2))        
    else
        error(['Nonphysical u: u = ' num2str(u) ' u >= ' num2str(1/beta + sqrt((alpha-beta)/(alpha*beta^2))) ...
            ' or u <= ' num2str(1/beta - sqrt((alpha-beta)/(alpha*beta^2)))]);
    end
end    
sigma_x = 1/2*sqrt(1/(sigma-beta*u^2/2));
sigma_y = 1/2*sqrt(1/(sigma-alpha*u^2/2));
delta_x = sqrt(1/2)*sqrt(1/(1/(4*alpha)+beta*u^2/4));
delta_y = sqrt(1/2)*sqrt(1/(1/(4*beta)+alpha*u^2/4));
% ---------------------------- %

% ----- TAGSM Theory ----- %
Sthy = TAGSM_W(sigma_x,sigma_y,delta_x,delta_y,u,xsrc,ysrc,xsrc,ysrc);
Wx1y2thy = TAGSM_W(sigma_x,sigma_y,delta_x,delta_y,u,x1,0,0,y2);
Wy1x2thy = TAGSM_W(sigma_x,sigma_y,delta_x,delta_y,u,0,y1,x2,0);
Wx1x2thy = TAGSM_W(sigma_x,sigma_y,delta_x,delta_y,u,x1a,sigma_y,x2a,sigma_y);
Wx1y1thy = TAGSM_W(sigma_x,sigma_y,delta_x,delta_y,u,x1a,y1a,sigma_x,sigma_y);
% ------------------------ %

% ----- TAGSM Simulation ----- %
h = exp(-2*pi*u*(alpha*repmat(reshape(ys(:),[1 1 M^2]),[M M 1]).*fxs ...
    - beta*repmat(reshape(xs(:),[1 1 M^2]),[M M 1]).*fys));
h(h == inf) = max(h(isfinite(h)));
h(h == -inf) = min(h(isfinite(h)));
Phi = exp(-1/2*(2*pi)^2*(alpha*fxs.^2 + beta*fys.^2));

numscreens = 100e3;
S = zeros(N);
Wx1y2 = zeros(N);
Wy1x2 = zeros(N);
Wx1x2 = zeros(N);
Wx1y1 = zeros(N);
for nn = 1:numscreens
    disp(nn);
    r = repmat(randn(M) + 1j*randn(M),[1 1 M^2]);
    R = r.*Phi;
    T = 2*pi/sqrt(L^2)*(alpha*beta)^(1/4)/sqrt(2*pi) ...
        *fftshift(fft(fftshift( fftshift(fft(fftshift(R.*h,2),[],2),2) ,1),[],1),1);
    for ii = 1:M
        ind = (ii-1)*M+1:ii*M;
        Tt = diag(squeeze(T(:,ii,ind)));
        Tout(:,ii) = Tt;
    end
    U = interp2(xs,ys,exp(-sigma*(xs.^2+ys.^2)).*Tout,xsrc,ysrc,'spline');
    S = S + abs(U).^2;
    Scorr(nn) = corr2(S/nn,Sthy);
    Wx1y2 = Wx1y2 + transpose(U(N/2+1,:))*(U(:,N/2+1)');
    Wy1x2 = Wy1x2 + U(:,N/2+1)*conj(U(N/2+1,:));
    Ui = interp2(xsrc,ysrc,U,xsrc,sigma_y*ones(N),'spline');
    Wx1x2 = Wx1x2 + transpose(Ui(N/2+1,:))*conj(Ui(N/2+1,:));
    Wx1y1 = Wx1y1 + U.*conj(interp2(xsrc,ysrc,U,sigma_x,sigma_y,'spline'));
end
S = S/numscreens;
Wx1y2 = transpose(Wx1y2/numscreens);
Wy1x2 = transpose(Wy1x2/numscreens);
Wx1x2 = transpose(Wx1x2/numscreens);
Wx1y1 = Wx1y1/numscreens;
% ---------------------------- %

save(['TAGSM_results_' num2str(numscreens) '.mat']);

figure; 
subplot(2,2,1);
imagesc(xsrc(1,:)*1000,ysrc(:,1)*1000,Sthy); axis image; colorbar;
subplot(2,2,2);
imagesc(xsrc(1,:)*1000,ysrc(:,1)*1000,S); axis image; colorbar;
subplot(2,2,3);
plot(xsrc(1,:)*1000,Sthy(N/2+1,:));
hold on;
plot(xsrc(1,:)*1000,S(N/2+1,:));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,2,4);
plot(ysrc(:,1)*1000,Sthy(:,N/2+1));
hold on;
plot(ysrc(:,1)*1000,S(:,N/2+1));
axis tight; grid on;
legend('Thy','Sim');

figure;
subplot(2,4,1);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wx1y2thy)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,2);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wx1y2)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,3);
plot(x1(1,:)*1000,real(Wx1y2thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,real(Wx1y2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,4);
plot(y2(:,1)*1000,real(Wx1y2thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,real(Wx1y2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,5);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wx1y2thy)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,6);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wx1y2)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,7);
plot(x1(1,:)*1000,imag(Wx1y2thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,imag(Wx1y2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,8);
plot(y2(:,1)*1000,imag(Wx1y2thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,imag(Wx1y2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');

figure;
subplot(2,4,1);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wy1x2thy)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,2);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wy1x2)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,3);
plot(x1(1,:)*1000,real(Wy1x2thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,real(Wy1x2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,4);
plot(y2(:,1)*1000,real(Wy1x2thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,real(Wy1x2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,5);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wy1x2thy)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,6);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wy1x2)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,7);
plot(x1(1,:)*1000,imag(Wy1x2thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,imag(Wy1x2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,8);
plot(y2(:,1)*1000,imag(Wy1x2thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,imag(Wy1x2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');

figure;
subplot(2,4,1);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wx1x2thy)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,2);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wx1x2)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,3);
plot(x1(1,:)*1000,real(Wx1x2thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,real(Wx1x2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,4);
plot(y2(:,1)*1000,real(Wx1x2thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,real(Wx1x2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,5);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wx1x2thy)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,6);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wx1x2)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,7);
plot(x1(1,:)*1000,imag(Wx1x2thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,imag(Wx1x2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,8);
plot(y2(:,1)*1000,imag(Wx1x2thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,imag(Wx1x2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');

figure;
subplot(2,4,1);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wx1y1thy)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,2);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wx1y1)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,3);
plot(x1(1,:)*1000,real(Wx1y1thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,real(Wx1y1(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,4);
plot(y2(:,1)*1000,real(Wx1y1thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,real(Wx1y1(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,5);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wx1y1thy)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,6);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wx1y1)); axis image; colorbar;
axis([-1 1 -1 1]*0.75);
subplot(2,4,7);
plot(x1(1,:)*1000,imag(Wx1y1thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,imag(Wx1y1(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,8);
plot(y2(:,1)*1000,imag(Wx1y1thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,imag(Wx1y1(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');

